/*
 * RELAY DEBUGFS defines and function prototypes
 *
 * Copyright (C) 2007 IBM Inc.
 *
 *	Tom Zanussi <zanussi@us.ibm.com>
 *	Martin Hunt <hunt@redhat.com>
 *	David Wilder <dwilder@us.ibm.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#ifndef _LINUX_RELAY_DEBUGFS_H
#define _LINUX_RELAY_DEBUGFS_H

#include <linux/relay.h>

/*
 * RELAY DEBUGFS channel flags
 */
#define RELAY_GLOBAL_CHANNEL	0x01
#define RELAY_FLIGHT_CHANNEL	0x02
#define RELAY_DISABLE_STATE	0x04

enum relay_state {
	RELAY_SETUP,
	RELAY_RUNNING,
	RELAY_STOPPED,
};

enum relay_dir_state {
	RELAY_PARENT_DIR_ABSENT,
	RELAY_PARENT_DIR_EXISTS,
	RELAY_DIR_EXISTS
};

#define RELAY_ROOT_NAME_SIZE	64	/* Max root dir identifier */
#define RELAY_NAME_SIZE		64	/* Max relay identifier */

/*
 * Buffers for use by relay_printk
 */
#define DEFAULT_RELAY_BUF_SIZE 4096
#define DEFAULT_RELAY_SUB_BUF_NR 40
/*
 * Global root user information
 */
struct relay_root {
	struct list_head list;
	char name[RELAY_ROOT_NAME_SIZE];
	struct dentry *root;
	unsigned int users;
};

/*
 * Client information
 */
struct relay_info {
	struct mutex state_mutex; /* Used to protect state changes */
	enum relay_state state;
	struct dentry *state_file;
	struct rchan *rchan;
	struct dentry *dir;
	struct dentry *dropped_file;
	struct dentry *reset_consumed_file;
	struct dentry *nr_sub_file;
	struct dentry *sub_size_file;
	atomic_t dropped;
	struct relay_root *root;
	void *private_data;
	unsigned int flags;
	unsigned int buf_size;
	unsigned int buf_nr;
	spinlock_t relay_lock;
};

struct relay_printk_data {
	char *parent_dir;
	char *dir;
	int exists;
	int buf_size;
	int sub_buf_size;
	unsigned long flags;
	struct relay_info *ti;
};

/*
 * Information about every 'relay debugfs' directory
 */
struct relay_dir {
	struct list_head relay_dir_list;
	char relay_dir_name[RELAY_NAME_SIZE];
	struct dentry *relay_root;
	struct dentry *relay_dir;
	struct relay_info *ti;
};

#ifdef CONFIG_RELAY_DEBUGFS
static inline int relay_running(struct relay_info *relay)
{
	return relay->state == RELAY_RUNNING;
}
struct relay_info *relay_setup(const char *root, const char *name,
			       u32 buf_size, u32 buf_nr, u32 flags);
int relay_start(struct relay_info *relay);
int relay_stop(struct relay_info *relay);
void relay_cleanup(struct relay_info *relay);
int relay_exists(const char *parent_dir, const char *dir,
					struct relay_info **ti);
void relay_cleanup_all(const char *parent_dir);
int relay_printk(struct relay_printk_data *dpk, char *format, ...);
int relay_dump(struct relay_printk_data *dpk, const void *output,
							const int output_len);
#else
static inline struct relay_info *relay_setup(const char *root,
					     const char *name, u32 buf_size,
					     u32 buf_nr, u32 flags)
{
	return NULL;
}
static inline int relay_start(struct relay_info *relay) { return -EINVAL; }
static inline int relay_stop(struct relay_info *relay) { return -EINVAL; }
static inline int relay_running(struct relay_info *relay) { return 0; }
static inline void relay_cleanup(struct relay_info *relay) {}
static inline int relay_exists(const char *parent_dir, const char *dir,
				struct relay_info **ti)
{
	return -EINVAL;
}
static inline void relay_cleanup_all(const char *parent_dir) {}
static inline int relay_printk(struct relay_printk_data *dpk, char *format,
				...)
{
	return -EINVAL;
}
int relay_dump(struct relay_printk_data *dpk, const void *output,
							const int output_len)
{
	return -EINVAL;
}

#endif

#endif /* ifdef CONFIG_RELAY_DEBUGFS */
